@@ -28,6 +28,11 @@ bool DynBPFFilter::block(const ComboAddress& addr, const struct timespec& until)
   bool inserted = false;
   std::unique_lock<std::mutex> lock(d_mutex);
 
+  if (d_excludedSubnets.match(addr)) {
+    /* do not add a block for excluded subnets */
+    return inserted;
+  }
+
   const container_t::iterator it = d_entries.find(addr);
   if (it != d_entries.end()) {
     if (it->d_until < until) {
