@@ -63,6 +63,31 @@ void setupLuaBindingsDNSQuestion()
 
       return *dq.ednsOptions;
     });
+  g_lua.registerFunction<std::string(DNSQuestion::*)(void)>("getTrailingData", [](const DNSQuestion& dq) {
+      const char* message = reinterpret_cast<const char*>(dq.dh);
+      const uint16_t messageLen = getDNSPacketLength(message, dq.len);
+      const std::string tail = std::string(message + messageLen, dq.len - messageLen);
+      return tail;
+    });
+  g_lua.registerFunction<bool(DNSQuestion::*)(std::string)>("setTrailingData", [](DNSQuestion& dq, const std::string& tail) {
+      char* message = reinterpret_cast<char*>(dq.dh);
+      const uint16_t messageLen = getDNSPacketLength(message, dq.len);
+      const uint16_t tailLen = tail.size();
+      if(tailLen > (dq.size - messageLen)) {
+        return false;
+      }
+
+      /* Update length and copy data from the Lua string. */
+      dq.len = messageLen + tailLen;
+      if(tailLen > 0) {
+        tail.copy(message + messageLen, tailLen);
+      }
+      return true;
+    });
+
+  g_lua.registerFunction<std::string(DNSQuestion::*)()>("getServerNameIndication", [](const DNSQuestion& dq) {
+      return dq.sni;
+    });
 
   g_lua.registerFunction<void(DNSQuestion::*)(std::string)>("sendTrap", [](const DNSQuestion& dq, boost::optional<std::string> reason) {
 #ifdef HAVE_NET_SNMP
@@ -123,6 +148,27 @@ void setupLuaBindingsDNSQuestion()
   g_lua.registerFunction<void(DNSResponse::*)(std::function<uint32_t(uint8_t section, uint16_t qclass, uint16_t qtype, uint32_t ttl)> editFunc)>("editTTLs", [](const DNSResponse& dr, std::function<uint32_t(uint8_t section, uint16_t qclass, uint16_t qtype, uint32_t ttl)> editFunc) {
         editDNSPacketTTL((char*) dr.dh, dr.len, editFunc);
       });
+  g_lua.registerFunction<std::string(DNSResponse::*)(void)>("getTrailingData", [](const DNSResponse& dq) {
+      const char* message = reinterpret_cast<const char*>(dq.dh);
+      const uint16_t messageLen = getDNSPacketLength(message, dq.len);
+      const std::string tail = std::string(message + messageLen, dq.len - messageLen);
+      return tail;
+    });
+  g_lua.registerFunction<bool(DNSResponse::*)(std::string)>("setTrailingData", [](DNSResponse& dq, const std::string& tail) {
+      char* message = reinterpret_cast<char*>(dq.dh);
+      const uint16_t messageLen = getDNSPacketLength(message, dq.len);
+      const uint16_t tailLen = tail.size();
+      if(tailLen > (dq.size - messageLen)) {
+        return false;
+      }
+
+      /* Update length and copy data from the Lua string. */
+      dq.len = messageLen + tailLen;
+      if(tailLen > 0) {
+        tail.copy(message + messageLen, tailLen);
+      }
+      return true;
+    });
   g_lua.registerFunction<void(DNSResponse::*)(std::string)>("sendTrap", [](const DNSResponse& dr, boost::optional<std::string> reason) {
 #ifdef HAVE_NET_SNMP
       if (g_snmpAgent && g_snmpTrapsEnabled) {
@@ -130,4 +176,48 @@ void setupLuaBindingsDNSQuestion()
       }
 #endif /* HAVE_NET_SNMP */
     });
+
+#ifdef HAVE_DNS_OVER_HTTPS
+    g_lua.registerFunction<std::string(DNSQuestion::*)(void)>("getHTTPPath", [](const DNSQuestion& dq) {
+      if (dq.du == nullptr) {
+        return std::string();
+      }
+      return dq.du->getHTTPPath();
+    });
+
+    g_lua.registerFunction<std::string(DNSQuestion::*)(void)>("getHTTPQueryString", [](const DNSQuestion& dq) {
+      if (dq.du == nullptr) {
+        return std::string();
+      }
+      return dq.du->getHTTPQueryString();
+    });
+
+    g_lua.registerFunction<std::string(DNSQuestion::*)(void)>("getHTTPHost", [](const DNSQuestion& dq) {
+      if (dq.du == nullptr) {
+        return std::string();
+      }
+      return dq.du->getHTTPHost();
+    });
+
+    g_lua.registerFunction<std::string(DNSQuestion::*)(void)>("getHTTPScheme", [](const DNSQuestion& dq) {
+      if (dq.du == nullptr) {
+        return std::string();
+      }
+      return dq.du->getHTTPScheme();
+    });
+
+    g_lua.registerFunction<std::unordered_map<std::string, std::string>(DNSQuestion::*)(void)>("getHTTPHeaders", [](const DNSQuestion& dq) {
+      if (dq.du == nullptr) {
+        return std::unordered_map<std::string, std::string>();
+      }
+      return dq.du->getHTTPHeaders();
+    });
+
+    g_lua.registerFunction<void(DNSQuestion::*)(uint16_t statusCode, const std::string& body, const boost::optional<std::string> contentType)>("setHTTPResponse", [](DNSQuestion& dq, uint16_t statusCode, const std::string& body, const boost::optional<std::string> contentType) {
+      if (dq.du == nullptr) {
+        return;
+      }
+      dq.du->setHTTPResponse(statusCode, body, contentType ? *contentType : "");
+    });
+#endif /* HAVE_DNS_OVER_HTTPS */
 }
