@@ -10,6 +10,7 @@
 
 Rings g_rings;
 GlobalStateHolder<NetmaskTree<DynBlock>> g_dynblockNMG;
+GlobalStateHolder<SuffixMatchTree<DynBlock>> g_dynblockSMT;
 
 BOOST_AUTO_TEST_SUITE(dnsdistdynblocks_hh)
 
@@ -40,7 +41,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QueryRate) {
        this should not trigger the rule */
     size_t numberOfQueries = 45 * numberOfSeconds;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -48,8 +49,8 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QueryRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) == nullptr);
   }
 
@@ -58,7 +59,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QueryRate) {
        this should trigger the rule this time */
     size_t numberOfQueries = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -66,16 +67,16 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QueryRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
     const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
     BOOST_CHECK_EQUAL(block.reason, reason);
-    BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+    BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
     BOOST_CHECK(block.domain.empty());
     BOOST_CHECK(block.action == action);
-    BOOST_CHECK_EQUAL(block.blocks, 0);
+    BOOST_CHECK_EQUAL(block.blocks, 0U);
     BOOST_CHECK_EQUAL(block.warning, false);
   }
 
@@ -108,7 +109,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QTypeRate) {
        this should not trigger the rule */
     size_t numberOfQueries = 45 * numberOfSeconds;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -116,8 +117,8 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QTypeRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) == nullptr);
   }
 
@@ -126,7 +127,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QTypeRate) {
        but for the wrong QType */
     size_t numberOfQueries = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -134,8 +135,8 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QTypeRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) == nullptr);
   }
 
@@ -144,7 +145,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QTypeRate) {
     // this should trigger the rule this time
     size_t numberOfQueries = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -152,16 +153,16 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_QTypeRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
     const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
     BOOST_CHECK_EQUAL(block.reason, reason);
-    BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+    BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
     BOOST_CHECK(block.domain.empty());
     BOOST_CHECK(block.action == action);
-    BOOST_CHECK_EQUAL(block.blocks, 0);
+    BOOST_CHECK_EQUAL(block.blocks, 0U);
     BOOST_CHECK_EQUAL(block.warning, false);
   }
 
@@ -197,7 +198,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_RCodeRate) {
        this should not trigger the rule */
     size_t numberOfResponses = 45 * numberOfSeconds;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     dh.rcode = rcode;
@@ -206,8 +207,8 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_RCodeRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), numberOfResponses);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) == nullptr);
   }
 
@@ -215,7 +216,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_RCodeRate) {
     /* insert just above 50 FormErr/s from a given client in the last 10s */
     size_t numberOfResponses = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     dh.rcode = RCode::FormErr;
@@ -224,8 +225,8 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_RCodeRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), numberOfResponses);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) == nullptr);
   }
 
@@ -234,7 +235,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_RCodeRate) {
        this should trigger the rule this time */
     size_t numberOfResponses = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     dh.rcode = rcode;
@@ -243,16 +244,16 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_RCodeRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), numberOfResponses);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
     const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
     BOOST_CHECK_EQUAL(block.reason, reason);
-    BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+    BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
     BOOST_CHECK(block.domain.empty());
     BOOST_CHECK(block.action == action);
-    BOOST_CHECK_EQUAL(block.blocks, 0);
+    BOOST_CHECK_EQUAL(block.blocks, 0U);
     BOOST_CHECK_EQUAL(block.warning, false);
   }
 
@@ -288,7 +289,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_ResponseByteRate) {
        this should not trigger the rule */
     size_t numberOfResponses = 99 * numberOfSeconds;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     dh.rcode = rcode;
@@ -297,8 +298,8 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_ResponseByteRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), numberOfResponses);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) == nullptr);
   }
 
@@ -306,7 +307,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_ResponseByteRate) {
     /* insert just above 100 answers of 100 bytes per second from a given client in the last 10s */
     size_t numberOfResponses = 100 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     dh.rcode = rcode;
@@ -315,16 +316,16 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_ResponseByteRate) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfResponseEntries(), numberOfResponses);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
     const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
     BOOST_CHECK_EQUAL(block.reason, reason);
-    BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+    BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
     BOOST_CHECK(block.domain.empty());
     BOOST_CHECK(block.action == action);
-    BOOST_CHECK_EQUAL(block.blocks, 0);
+    BOOST_CHECK_EQUAL(block.blocks, 0U);
     BOOST_CHECK_EQUAL(block.warning, false);
   }
 
@@ -357,7 +358,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
        this should not trigger the rule */
     size_t numberOfQueries = 20 * numberOfSeconds;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -366,7 +367,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
     dbrg.apply(now);
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 0U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) == nullptr);
   }
 
@@ -375,7 +376,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
        this should trigger the warning rule this time */
     size_t numberOfQueries = 20 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -384,17 +385,17 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
     dbrg.apply(now);
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
 
     {
       const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
       BOOST_CHECK_EQUAL(block.reason, reason);
-      BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+      BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
       BOOST_CHECK(block.domain.empty());
       BOOST_CHECK(block.action == DNSAction::Action::NoOp);
-      BOOST_CHECK_EQUAL(block.blocks, 0);
+      BOOST_CHECK_EQUAL(block.blocks, 0U);
       BOOST_CHECK_EQUAL(block.warning, true);
       /* let's increment the number of blocks so we can check that the counter
          is preserved when the block is upgraded to a non-warning one */
@@ -404,7 +405,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
     /* now inserts 50 qps for the same duration, we should reach the blocking threshold */
     numberOfQueries = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
       g_rings.insertQuery(now, requestor1, qname, qtype, size, dh);
@@ -412,18 +413,18 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
     dbrg.apply(now);
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
 
     {
       const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
       BOOST_CHECK_EQUAL(block.reason, reason);
-      BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+      BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
       BOOST_CHECK(block.domain.empty());
       BOOST_CHECK(block.action == action);
       /* this hsould have been preserved */
-      BOOST_CHECK_EQUAL(block.blocks, 1);
+      BOOST_CHECK_EQUAL(block.blocks, 1U);
       BOOST_CHECK_EQUAL(block.warning, false);
       block.blocks++;
     }
@@ -433,7 +434,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
     now.tv_sec += 30;
     numberOfQueries = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
       g_rings.insertQuery(now, requestor1, qname, qtype, size, dh);
@@ -441,7 +442,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
     dbrg.apply(now);
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
 
@@ -449,11 +450,11 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
       const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
       BOOST_CHECK_EQUAL(block.reason, reason);
       /* should have been updated */
-      BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+      BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
       BOOST_CHECK(block.domain.empty());
       BOOST_CHECK(block.action == action);
       /* this hsould have been preserved */
-      BOOST_CHECK_EQUAL(block.blocks, 2);
+      BOOST_CHECK_EQUAL(block.blocks, 2U);
       BOOST_CHECK_EQUAL(block.warning, false);
     }
   }
@@ -463,7 +464,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
        this should trigger the blocking rule right away this time */
     size_t numberOfQueries = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -472,17 +473,17 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Warning) {
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries);
 
     dbrg.apply(now);
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
 
     {
       const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
       BOOST_CHECK_EQUAL(block.reason, reason);
-      BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+      BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
       BOOST_CHECK(block.domain.empty());
       BOOST_CHECK(block.action == action);
-      BOOST_CHECK_EQUAL(block.blocks, 0);
+      BOOST_CHECK_EQUAL(block.blocks, 0U);
       BOOST_CHECK_EQUAL(block.warning, false);
     }
   }
@@ -519,7 +520,7 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Ranges) {
        this should trigger the rule for the first one but not the second one */
     size_t numberOfQueries = 50 * numberOfSeconds + 1;
     g_rings.clear();
-    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0);
+    BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), 0U);
     g_dynblockNMG.setState(emptyNMG);
 
     for (size_t idx = 0; idx < numberOfQueries; idx++) {
@@ -528,16 +529,16 @@ BOOST_AUTO_TEST_CASE(test_DynBlockRulesGroup_Ranges) {
     }
     BOOST_CHECK_EQUAL(g_rings.getNumberOfQueryEntries(), numberOfQueries * 2);
 
-    dbrg.apply();
-    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1);
+    dbrg.apply(now);
+    BOOST_CHECK_EQUAL(g_dynblockNMG.getLocal()->size(), 1U);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor1) != nullptr);
     BOOST_CHECK(g_dynblockNMG.getLocal()->lookup(requestor2) == nullptr);
     const auto& block = g_dynblockNMG.getLocal()->lookup(requestor1)->second;
     BOOST_CHECK_EQUAL(block.reason, reason);
-    BOOST_CHECK_EQUAL(block.until.tv_sec, now.tv_sec + blockDuration);
+    BOOST_CHECK_EQUAL(static_cast<size_t>(block.until.tv_sec), now.tv_sec + blockDuration);
     BOOST_CHECK(block.domain.empty());
     BOOST_CHECK(block.action == action);
-    BOOST_CHECK_EQUAL(block.blocks, 0);
+    BOOST_CHECK_EQUAL(block.blocks, 0U);
     BOOST_CHECK_EQUAL(block.warning, false);
   }
 
